/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var util = require('util');
var upnp = require('./upnp');
var xml = require('../xml');
var cfg = require('../cfg');
var fmt = require('../fmt');

//-----------------------------------------------------------------------------

var ns = {
  av:       'urn:schemas-upnp-org:av:av',
  dc:       'http://purl.org/dc/elements/1.1/',
  upnp:     'urn:schemas-upnp-org:metadata-1-0/upnp/',
  didl:     'urn:schemas-upnp-org:metadata-1-0/DIDL-Lite/',
  dlna:     'urn:schemas-dlna-org:metadata-1-0/',
  wmpns:    'urn:schemas-microsoft-com:WMPNSS-1-0/',
  upnp_dev: 'urn:schemas-upnp-org:device-1-0',
  dlna_dev: 'urn:schemas-dlna-org:device-1-0',
  pnpx:     'http://schemas.microsoft.com/windows/pnpx/2005/11'
};

//-----------------------------------------------------------------------------

var id = {
  id:              'id',
  parentID:        'parentID',
  title:           'dc:title',
  date:            'dc:date',
  creator:         'dc:creator',
  mediaClass:      'av:mediaClass',
  writeStatus:     'writeStatus',
  restricted:      'restricted',
  upnpClass:       'upnp:class',
  album:           'upnp:album',
  artist:          'upnp:artist',
  genre:           'upnp:genre',
  track:           'upnp:originalTrackNumber',
  // Container properties
  childCount:      'childCount',
  searchable:      'searchable',
  hideCount:       'hideCount',
  multiItem:       'multiItem', // for sorting
  // Resource properties
  res:             'res', // resource array
  url:             'url',
  protocolInfo:    'protocolInfo',
  resolution:      'resolution',
  colorDepth:      'colorDepth',
  size:            'size',
  duration:        'duration',
  bitrate:         'bitrate',
  sampleFrequency: 'sampleFrequency',
  nrAudioChannels: 'nrAudioChannels',
  // Album Art
  upnpAlbumArtURI: 'upnp:albumArtURI',
  dlnaProfileID:   'dlna:profileID'
};

//-----------------------------------------------------------------------------
// Base Item
//-----------------------------------------------------------------------------

function BaseItem(uid, pid, title) {
  this[id.id] = uid;
  this[id.parentID] = pid;
  this[id.title] = title;
  this[id.restricted] = true;
  this[id.res] = [];
}

BaseItem.prototype.getDate = function() {
  return this[id.date] || 0;
};

BaseItem.prototype.setDate = function(date) {
  this[id.date] = date;
};

BaseItem.prototype.getTitle = function() {
  return this[id.title];
};

BaseItem.prototype.setTitle = function(title) {
  this[id.title] = title;
};

BaseItem.prototype.getMediaClass = function() {
  return this[id.mediaClass];
};

BaseItem.prototype.setMediaClass = function(mediaClass) {
  this[id.mediaClass] = mediaClass;
};

BaseItem.prototype.getUpnpClass = function() {
  return this[id.upnpClass];
};

BaseItem.prototype.setUpnpClass = function(upnpClass) {
  this[id.upnpClass] = upnpClass;
};

BaseItem.prototype.getAlbum = function() {
  return this[id.album];
};

BaseItem.prototype.setAlbum = function(album) {
  this[id.album] = album;
};

BaseItem.prototype.getGenre = function() {
  return this[id.genre];
};

BaseItem.prototype.setGenre = function(genre) {
  this[id.genre] = genre;
};

BaseItem.prototype.getArtist = function() {
  return this[id.artist];
};

BaseItem.prototype.setArtist = function(artist) {
  this[id.artist] = artist;
};

BaseItem.prototype.getCreator = function() {
  return this[id.creator];
};

BaseItem.prototype.setCreator = function(creator) {
  this[id.creator] = creator;
};

BaseItem.prototype.setAuthor = function(author) {
  this.setArtist(author);
  this.setCreator(author);
};

BaseItem.prototype.getTrack = function() {
  return this[id.track];
};

BaseItem.prototype.setTrack = function(track) {
  this[id.track] = track;
};

BaseItem.prototype.addResource = function(res) {
  if (res) {
    this[id.res].push(res);
  }
};

BaseItem.prototype.prependResource = function(res) {
  if (res) {
    this[id.res].unshift(res);
  }
};

BaseItem.prototype.fillXobj = function(xobj) {
  setAttr(this, xobj, id.title);
  setAttr(this, xobj, id.date, dcDate);
  setAttr(this, xobj, id.creator);
  setAttr(this, xobj, id.upnpClass);
  setAttr(this, xobj, id.album);
  setAttr(this, xobj, id.genre);
  setAttr(this, xobj, id.artist);
  setAttr(this, xobj, id.track);
  setAttr(this, xobj, id.mediaClass);
  var res = this[id.res];
  var len = res.length;
  if (len > 0) {
    for (var i = 0; i < len; i++) {
      res[i] = res[i].toXJSON();
    }
    xobj['#list'] = res;
  }
};

//-----------------------------------------------------------------------------
// Container
//-----------------------------------------------------------------------------

function Container(uid, pid, title, count) {
  BaseItem.call(this, uid, pid, title);
  this[id.upnpClass]  = upnp.STORAGE_FOLDER_CONTAINER;
  this[id.childCount] = count || 0;
  this[id.searchable] = false;
  this[id.hideCount]  = false;
  this[id.multiItem]  = false;
}

util.inherits(Container, BaseItem);

Container.prototype.toXJSON = function() {
  var self = this;
  var xobj = {
    container: {
      '@': {
        id:         self[id.id],
        parentID:   self[id.parentID],
        childCount: self[id.childCount],
        searchable: self[id.searchable],
        restricted: self[id.restricted]
      }
    }
  };
  self.fillXobj(xobj.container);
  if (cfg.childCount() && !self[id.hideCount]) {
    xobj.container[id.title] += ' [' + fmt.thousandSeparator(self[id.childCount]) + ']';
  }
  return xobj;
};

Container.prototype.setChildCount = function(count) {
  this[id.childCount] = count;
};

Container.prototype.setHideCount = function(hide) {
  this[id.hideCount] = hide;
};

Container.prototype.addChildCount = function(count) {
  this[id.childCount] += count;
};

Container.prototype.isMultiItem = function() {
  return this[id.multiItem];
};

Container.prototype.setMultiItem = function(multi) {
  this[id.multiItem] = multi;
};

exports.Container = Container;

//-----------------------------------------------------------------------------
// Item
//-----------------------------------------------------------------------------

function Item(uid, pid, title) {
  BaseItem.call(this, uid, pid, title);
}

util.inherits(Item, BaseItem);

Item.prototype.toXJSON = function() {
  var self = this;
  var xobj = {
    item: {
      '@': {
        id:         self[id.id],
        parentID:   self[id.parentID],
        restricted: self[id.restricted]
      }
    }
  };
  self.fillXobj(xobj.item);
  return xobj;
};

exports.Item = Item;

exports.imageItem = function(uid, pid, title) {
  var obj = new Item(uid, pid, title);
  obj.setUpnpClass(upnp.IMAGE_ITEM);
  obj.setMediaClass(upnp.AV_PHOTO);
  return obj;
};

exports.audioItem = function(uid, pid, title) {
  var obj = new Item(uid, pid, title);
  obj.setUpnpClass(upnp.AUDIO_ITEM);
  obj.setMediaClass(upnp.AV_MUSIC);
  return obj;
};

exports.videoItem = function(uid, pid, title) {
  var obj = new Item(uid, pid, title);
  obj.setUpnpClass(upnp.VIDEO_ITEM);
  obj.setMediaClass(upnp.AV_VIDEO);
  return obj;
};

//-----------------------------------------------------------------------------
// Resource
//-----------------------------------------------------------------------------

function Res(url, profile) {
  this[id.url] = url;
  this[id.protocolInfo] = (profile) ? profile.toString() : null;
}

Res.prototype.setProfile = function(profile) {
  this[id.protocolInfo] = profile;
};

Res.prototype.setResolution = function(width, height) {
  this[id.resolution] = width + 'x' + height;
};

Res.prototype.setColorDepth = function(colorDepth) {
  this[id.colorDepth] = colorDepth;
};

Res.prototype.setSize = function(size) {
  this[id.size] = size;
};

Res.prototype.setDuration = function(duration) {
  this[id.duration] = duration;
};

Res.prototype.setBitrate = function(bitrate) {
  this[id.bitrate] = bitrate;
};

Res.prototype.setSampleFrequency = function(sampleFrequency) {
  this[id.sampleFrequency] = sampleFrequency;
};

Res.prototype.setChannels = function(channels) {
  this[id.nrAudioChannels] = channels;
};

Res.prototype.toXJSON = function() {
  var self = this;
  var xobj = {
    res: {
      '@': {},
      '#text': self[id.url]
    }
  };
  var attr = xobj.res['@'];
  setAttr(self, attr, id.protocolInfo);
  setAttr(self, attr, id.size);
  setAttr(self, attr, id.duration, resDuration);
  setAttr(self, attr, id.bitrate);
  setAttr(self, attr, id.nrAudioChannels);
  setAttr(self, attr, id.resolution);
  setAttr(self, attr, id.colorDepth);
  return xobj;
};

exports.Res = Res;

//-----------------------------------------------------------------------------
// Album Art
//-----------------------------------------------------------------------------

function AlbumArt(url, profile) {
  this[id.url] = url;
  this[id.dlnaProfileID] = profile;
}

AlbumArt.prototype.toXJSON = function() {
  var xobj = {};
  var uart = {};
  var prof = this[id.dlnaProfileID];
  if (prof !== undefined && prof !== null) {
    var attr = {};
    attr[id.dlnaProfileID] = prof;
    uart['@'] = attr;
  }
  uart['#text'] = this[id.url];
  xobj[id.upnpAlbumArtURI] = uart;
  return xobj;
};

exports.AlbumArt = AlbumArt;

//-----------------------------------------------------------------------------

function didlXML(items) {

  var len = items.length;

  for (var i = 0; i < len; i++) {
    items[i] = items[i].toXJSON();
  }

  var doc = {
    'DIDL-Lite': {
      '@': {
        'xmlns':      ns.didl,
        'xmlns:dc':   ns.dc,
        'xmlns:av':   ns.av,
        'xmlns:upnp': ns.upnp,
        'xmlns:dlna': ns.dlna
      },
      '#list': items
    }
  };

  return xml.toXML(doc, false);

}

exports.didlXML = didlXML;

//-----------------------------------------------------------------------------
// Helpers
//-----------------------------------------------------------------------------

function setAttr(src, dst, name, proc) {
  var val = src[name];
  if (val !== undefined && val !== null) {
    dst[name] = (proc) ? proc(val) : val;
  }
}

function tpad(number) {
  if (number < 10) {
    return '0' + number;
  }
  return number;
}

function mpad(millis) {
  if (millis < 10) {
    return '00' + millis;
  }
  else if (millis < 100) {
    return '0' + millis;
  }
  return millis;
}

function dcDate(d) {

  if (typeof d === 'number') {
    d = new Date(d);
  }

  return d.getUTCFullYear() +
    '-' + tpad(d.getUTCMonth() + 1) +
    '-' + tpad(d.getUTCDate()) +
    'T' + tpad(d.getUTCHours()) +
    ':' + tpad(d.getUTCMinutes()) +
    ':' + tpad(d.getUTCSeconds());

}

function resDuration(d) {

  if (typeof d === 'number') {
    d = new Date(d);
  }

  return tpad(d.getUTCHours())
    + ':' + tpad(d.getUTCMinutes())
    + ':' + tpad(d.getUTCSeconds())
    + '.' + mpad(d.getUTCMilliseconds());

}