/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var fs     = require('fs');
var byline = require('byline');
var async  = require('async');

var WSTREAM_OPTS = { flags: 'w', encoding: 'utf8' };
var RSTREAM_OPTS = { flags: 'r', encoding: 'utf8' };
var FREAD_OPTS   = { encoding: 'utf8' };
var FWRITE_OPTS  = { encoding: 'utf8' };

//-----------------------------------------------------------------------------
// Save
//-----------------------------------------------------------------------------

function save(fpath, data, cb) {

  var keys = Object.keys(data);
  var klen = keys.length;

  if (klen === 0) {
    fs.unlink(fpath, cb);
    return;
  }

  var ws = fs.createWriteStream(fpath, WSTREAM_OPTS);

  async.eachSeries(keys,
    function(key, wcb) {
      var val = data[key];
      if (val === undefined) return wcb();
      var buf;
      try {
        buf = key + ',' + JSON.stringify(val) + '\n';
      }
      catch (e) {
        return wcb();
      }
      if (ws.write(buf)) {
        return wcb();
      }
      ws.once('drain', wcb);
    },
    function(err) {
      try { ws.end() } catch (e) {}
      cb(err);
    }
  );

}

exports.save = save;

//-----------------------------------------------------------------------------
// Load
//-----------------------------------------------------------------------------

function load(fpath, cb) {

  var rv = {};

  var sfile = fs.createReadStream(fpath, RSTREAM_OPTS);

  sfile.on('error', function() { cb(null, rv) });
  sfile.on('end', function() { cb(null, rv) });

  var sline = byline(sfile);

  sline.on('data', function(line) {
    var pos = line.indexOf(',');
    if (pos < 1) return;
    try {
      var key = line.substring(0, pos);
      rv[key] = JSON.parse(line.substring(pos + 1));
    } catch (e) {
      // ignore
    }
  });

}

exports.load = load;

//-----------------------------------------------------------------------------
// Pure JSON storage
//-----------------------------------------------------------------------------

function loadJSON(fpath, cb) {

  fs.readFile(fpath, FREAD_OPTS,
    function(err, text) {
      if (err) return cb(null, {});
      try {
        var obj = JSON.parse(text);
        cb(null, obj);
      } catch (e) {
        cb(null, {});
      }
    }
  );

}

function saveJSON(fpath, data, cb) {

  var text;

  try {
    text = JSON.stringify(data)
  }
  catch (e) {
    return cb(e);
  }

  fs.writeFile(fpath, text, FWRITE_OPTS, cb);

}

exports.loadJSON = loadJSON;
exports.saveJSON = saveJSON;

