/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */


'use strict';

var evt = require('../evt');
var env = require('../env');
var hnd = require('../hnd');
var log = require('../log');
var rti = require('../rti');
var htp = require('../htp');
var cfg = require('../cfg');
var mdb = require('./mdb');
var tmb = require('./tmb');
var img = require('./img');
var alg = require('./alg');
var prf = require('./prf');
var cdr = require('../cdr');
var didl = require('../cls/didl');
var flag = require('../cls/flags');
var ffm  = require('../ffm');
var enc = require('../menc/menc');

var id    = rti.id;
var mtype = rti.mtype;
var pmime = prf.mime;

var minfo = mdb.minfo;
var scale = mdb.scale;

var AUD_HND = 'aud';
var ART_HND = 'art';

var albumArt = [];

//-----------------------------------------------------------------------------
// Browse Handlers
//-----------------------------------------------------------------------------

function audioItem(brq, md, cb) {
  if (brq.mt !== mtype.audio || !rti.isAudio(md)) return;
  cb(null, makeAudioItem(brq, md));
  return true;
}

hnd.addMediaHandler(audioItem);

//-----------------------------------------------------------------------------
// Audio Item
//-----------------------------------------------------------------------------

function makeAudioItem(brq, md) {

  var title = rti.fname(md, cfg.hideExt());

  var iid = hnd.fsoid(mtype.audio, rti.uid(md));

  var aud = didl.audioItem(iid, brq.oid, title);

  aud.setDate(md[id.mtime]);

  setMeta(md, aud);

  aud.addResource(audioRes(brq, md));

  setArt(brq, md, aud);

//  aud.addResource(thumbRes(brq, md));

  return aud;

}

function audioRes(brq, md) {
  var url = hnd.streamURL(brq.surl, AUD_HND, rti.uid(md));
  var opt = getOptions(brq.dev, md);
  var pro = opt.pro;
  var res = new didl.Res(url, pro);
  res.setDuration(md[id.duration]);
  res.setBitrate(md[id.bitrate]);
  if (!pro.getConv()) {
    res.setSize(md[id.fsize]);
  }
  return res;
}

function setMeta(md, aud) {
  var val = md[id.album];
  if (rti.notNull(val)) aud.setAlbum(val);
  val = md[id.performer];
  if (rti.notNull(val)) aud.setAuthor(val);
  val = md[id.genre];
  if (rti.notNull(val)) aud.setGenre(val);
}

function setArt(brq, md, aud) {

  var amd = null;

  var arts = brq.dev.arts;
  var artp = brq.dev.artp;

  if (arts[0] === 0) return;

  var alen = albumArt.length;

  if (alen > 0) {
    for (var i = 0; i < alen; i++) {
      amd = brq.image[albumArt[i]];
      if (amd) break;
    }
  }

  if (!amd) amd = md;

  var aurl = hnd.streamURL(brq.surl, ART_HND, rti.uid(amd));

  if (artp === 'none') artp = null;

  aud.prependResource(new didl.AlbumArt(aurl, artp));

}

//-----------------------------------------------------------------------------
// Aduio Stream Handlers
//-----------------------------------------------------------------------------

function audioStreamHandler(srq) {

  var uid = srq.uid;
  var req = srq.req;
  var res = srq.res;
  var md  = minfo.get(uid);

  if (!md) {
    log.warn('Audio media data not in cache: %s', uid);
    res.sendStatus(404);
    return;
  }

  var opt = getOptions(srq.dev, md);
  var pro = opt.pro;

  srq.prf = pro.toString(true);

  htp.setContentType(res, pro.getMime());

  htp.setDlnaHeaders(srq, htp.TRANSFER_STREAMING);

  if (!pro.getConv()) {
    evt.postStartStream(req.ip);
    res.sendFile(rti.fpath(md), htp.SEND_FILE_OPT(), evt.stopStreamCb(req.ip));
    return;
  }

  // Transcoding

  var time = htp.getTimeRange(req, opt.len);

  htp.setTimeRange(res, srq.dev, time);

  res.set('Connection', 'close');
  res.set('Transfer-Encoding', 'chunked');

  if (time) {
    res.status(206);
  }

  if (req.method === 'HEAD') {
    res.end();
    return;
  }

  // Init options

  opt.res = res;
  opt.ecb = evt.stopStreamCb(req.ip);
  opt.pos = (time) ? time.start : 0;
  opt.url = rti.fpath(md);

  evt.postStartStream(req.ip);

  cdr.run(opt);

}

hnd.addStreamHandler(AUD_HND, audioStreamHandler);

//-----------------------------------------------------------------------------
// Art Stream Handler
//-----------------------------------------------------------------------------

function artStreamHandler(srq) {

  var uid = srq.uid;
  var res = srq.res;
  var md  = minfo.get(uid);

  if (!md) {
    log.warn('Album art not in cache: %s', uid);
    res.sendStatus(404);
    return;
  }

  srq.prf = prf.TMB_SHORT;

  htp.setDlnaHeaders(srq, htp.TRANSFER_INTERACTIVE);

  if (!rti.isImage(md)) {
    tmb.sendThumb(md, srq.req, res);
    return;
  }

  var w = md[id.width];
  var h = md[id.height];
  var m = md[id.fmime];

  var arts = srq.dev.arts;

  var conv = false;

  if (w > arts[0] || h > arts[1]) {
    var wh = alg.scale(w, h, arts[0], arts[1]);
    w = wh[0];
    h = wh[1];
    conv = true;
  }
  else if (m !== pmime.jpeg) {
    conv = true;
  }

  if (conv) {
    tmb.sendScale(md, w, h, res);
  }
  else {
    res.sendFile(rti.fpath(md), htp.SEND_FILE_OPT());
  }

}

hnd.addStreamHandler(ART_HND, artStreamHandler);

//-----------------------------------------------------------------------------
// Audio Profiles
//-----------------------------------------------------------------------------

function getOptions(dev, md) {

  var opt = new ffm.Options();

  var acode = dev.acode;

  if (!rti.isString(acode) || acode === 'none' || enc.isNative(dev, md)) {
    opt.pro = getNativeProfile(dev, md);
    return opt;
  }

  opt.len = rti.duration(md);
  opt.aud = rti.audioTrack(md, 0);

  hnd.getProfile(acode, dev, opt);

  if (!opt.pro) {
    log.warn('Audio transcoding not found for: %s (%s)', acode, rti.fpath(md));
    opt.pro = getNativeProfile(dev, md);
  }

  return opt;

}

function getNativeProfile(dev, md) {

  var fmime = rti.fmime(md);
  var track = rti.audioTrack(md);
  var chann = rti.channels(track);
  var pname = null;

  switch (fmime) {
    case pmime.mp3:
      pname = 'MP3';
      break;
    case pmime.mp4a:
      pname = (chann < 5) ? 'AAC_ISO' : 'AAC_MULT5_ISO';
      break;
    case pmime.wma:
      pname = (chann < 5) ? 'WMAFULL' : 'WMAPRO';
      break;
    case pmime.wav:
      pname = 'WAVE';
      break;
    case pmime.flac:
      pname = 'FLAC';
      break;
    case pmime.ogg:
      pname = 'OGG';
      break;
    case pmime.adts:
      pname = 'AAC_ADTS';
      break;
    case pmime.ac3:
      pname = 'AC3';
      break;
    case pmime.dts:
      pname = 'DTS';
      break;
  }

  fmime = dev.mime[rti.codecid(md)] || fmime;

  return new prf.Profile(pname, fmime, flag.SBYTE, flag.op.range, flag.ci.none);

}

//-----------------------------------------------------------------------------
// Config
//-----------------------------------------------------------------------------

function onConfig() {
  albumArt = cfg.getAlbumArt();
}

evt.onConfigReadyChange(onConfig);