/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// File System Browser

'use strict';

var async = require('async');
var npath = require('path');
var fs = require('fs');

var hnd = require('../hnd');
var log = require('../log');
var rti = require('../rti');
var cfg = require('../cfg');
var fop = require('../fop');

var mdb = require('./mdb');
var mdi = require('./mdi');
var srt = require('./srt');

var didl = require('../cls/didl');
var Container = didl.Container;

var id = rti.id;

var mtype = rti.mtype;
var minfo = mdb.minfo;
var parents = mdb.parents;

//-----------------------------------------------------------------------------

function filterNull(val, cb) {
  cb(val !== null);
}

//-----------------------------------------------------------------------------

function classifyMedia(brq, md) {

  var fname = md[id.fname].toLowerCase();

  if (rti.isFolder(md)) {
    brq.dir[fname] = md;
    return;
  }

  var fextn = md[id.fextn];

  if (fextn !== undefined && fextn !== null) {
    fname += '.' + fextn.toLowerCase();
  }

  switch (md[id.mtype]) {
    case mtype.image:
      brq.image[fname] = md;
      break;
    case mtype.audio:
      brq.audio[fname] = md;
      break;
    case mtype.video:
      brq.video[fname] = md;
      break;
    case mtype.text:
      brq.text[fname] = md;
      break;
    default:
      brq.other[fname] = md;
      break;
  }

}

//-----------------------------------------------------------------------------

function fileBrowser(brq, cb) {
  var uid = brq.uid;
  brq.mt = parseInt(uid.charAt(0), 10);
  uid = uid.substring(1);
  var md = minfo.get(uid);
  if (!md) {
    return cb('Media info not found for: ' + brq.oid);
  }
  if (brq.meta) {
    return fileMeta(brq, md, cb);
  }
  brq.uid = uid;
  var fpath = rti.fpath(md);
  if (!rti.isFolder(md)) {
    //log.warn('Browse request for a file: %s', fpath);
    return cb();
  }
  var rootDir = findTopFolder(brq.mt, uid);
  if (!rootDir) {
    log.warn('Root folder not found for: %s', fpath);
  }
  var sortProc = srt.getSortProc((rootDir) ? rootDir.sort : rti.sort.name);
  var nodirs = (rootDir) ? rootDir.nodirs : false;
  log.trace('Reading folder: %s', fpath);
  fs.readdir(fpath, function(err, fnames) {
    if (err) return cb(err);
    var len = fnames.length;
    if (len === 0) return cb();
    log.trace('Getting children media data: %s', fpath);
    async.map(fnames,
      function(name, mdcb) {
        if (name.charAt(0) === '.') return mdcb(null, null);
        mdi.fileInfo(npath.join(fpath, name), 0, true, mdcb);
      },
      function(err, mdary) {
        if (err) return cb(err);
        if (mdary.length === 0) return cb();
        log.trace('Filtering media data: %s', fpath);
        async.filter(mdary,
          function(fmd, fcb) {
            if (fmd === null || rti.isUnknownFileType(fmd) || rti.isHidden(fmd)) {
              fcb(false);
            }
            else if (nodirs && rti.isFolder(fmd)) {
              fcb(false);
            }
            else {
              fcb(true);
            }
          },
          function(mdflt) {
            var len = mdflt.length;
            if (len === 0) return cb();
            for (var i = 0; i < len; i++) {
              classifyMedia(brq, mdflt[i]);
            }
            log.trace('Processing media data: %s', fpath);
            async.map(mdflt,
              function(xmd, xcb) {
                if (rti.isFolder(xmd)) {
                  parents.set(rti.uid(xmd), uid);
                }
                hnd.processMedia(brq, xmd, xcb);
              },
              function(err, didlAry) {
                if (err) return cb(err);
                if (didlAry.length === 0) return cb();
                log.trace('Filtering DIDL items: %s', fpath);
                async.filter(didlAry, filterNull,
                  function(didlValid) {
                    if (didlValid.length === 0) return cb();
                    didlValid.sort(sortProc);
                    brq.ary = brq.ary.concat(didlValid);
                    cb();
                  }
                );
              }
            );
          }
        );
      }
    );
  });
}

hnd.addFileBrowser(fileBrowser);

//-----------------------------------------------------------------------------

function fileMeta(brq, md, cb) {
  hnd.processMedia(brq, md, function(err, item) {
    if (item) {
      brq.append(item);
    }
    cb();
  });
}

//-----------------------------------------------------------------------------
// Folder Handler
//-----------------------------------------------------------------------------

function folderMedia(brq, md, cb) {
  if (!rti.isFolder(md)) return;
  var countProc = (cfg.childCount()) ? fop.childCount : fop.dummyCount;
  countProc(rti.fpath(md), function(err, count) {
    if (count === 0 && cfg.hideEmpty()) return cb(null, null);
    var dir = new Container(hnd.fsoid(brq.mt, md[id.uid]), brq.oid, md[id.fname], count);
    dir.setDate(md[id.mtime]);
    cb(null, dir);
  });
  return true;
}

hnd.addMediaHandler(folderMedia);

//-----------------------------------------------------------------------------
// Top Folder Finder
//-----------------------------------------------------------------------------

function findTopFolder(mt, uid) {

  function getRoots() {
    switch (mt) {
      case mtype.image: return cfg.imageRoots();
      case mtype.audio: return cfg.audioRoots();
      case mtype.video: return cfg.videoRoots();
    }
    log.warn('findTopFolder roots not found for mediaType: %s', mt);
    return null;
  }

  var roots = getRoots();

  if (!roots) return null;

  var key = parents.findParentKeyForVal(uid, null);

  if (!key) return null;

  for (var fpath in roots) {
    var root = roots[fpath];
    if (key === root.uid) return root;
  }

  return null;

}