/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// Image File Streamer

'use strict';

var evt = require('../evt');
var hnd = require('../hnd');
var log = require('../log');
var rti = require('../rti');
var htp = require('../htp');
var cfg = require('../cfg');
var mdb = require('./mdb');
var tmb = require('./tmb');
var alg = require('./alg');
var prf = require('./prf');
var didl = require('../cls/didl');

var Res = didl.Res;

var id = rti.id;
var mtype = rti.mtype;
var mime = prf.mime;

var minfo = mdb.minfo;
var scale = mdb.scale;

var downscale;
var autorotate;

var TMB_HND = 'tmb';
var IMG_HND = 'img';

//-----------------------------------------------------------------------------
// Browse Handlers
//-----------------------------------------------------------------------------

function imageItem(brq, md, cb) {
  if (brq.mt !== mtype.image || !rti.isImage(md)) return;
  cb(null, makeImageItem(brq, md));
  return true;
}

hnd.addMediaHandler(imageItem);

//-----------------------------------------------------------------------------
// Image Item
//-----------------------------------------------------------------------------

function makeImageItem(brq, md) {

  var title = rti.fname(md, cfg.hideExt());

  var iid = hnd.fsoid(mtype.image, rti.uid(md));

  var img = didl.imageItem(iid, brq.oid, title);

  img.setDate(md[id.mtime]);

  var tc = tcodeParams(brq.dev, md);

  img.addResource(imageRes(brq, md, tc.w, tc.h));
  img.addResource(thumbRes(brq, md));

  return img;

}

exports.makeImageItem = makeImageItem;

//-----------------------------------------------------------------------------

function imageRes(brq, md, w, h) {
  var url = hnd.streamURL(brq.surl, IMG_HND, rti.uid(md));
  var res = new Res(url, prf.IMG_FULL);
  res.setColorDepth(24);
  res.setResolution(w, h);
  return res;
}

exports.imageRes = imageRes;

function thumbRes(brq, md) {
  if (!md[id.tvalid]) return null;
  var url = hnd.streamURL(brq.surl, TMB_HND, rti.uid(md));
  var res = new Res(url, prf.TMB_FULL);
  res.setColorDepth(24);
  res.setResolution(md[id.twidth], md[id.theight]);
  return res;
}

exports.thumbRes = thumbRes;

//-----------------------------------------------------------------------------
// Stream Handlers
//-----------------------------------------------------------------------------

function imageStream(srq, imagePath) {
  htp.setDlnaHeaders(srq, htp.TRANSFER_INTERACTIVE);
  evt.postStartStream();
  srq.res.sendFile(imagePath, htp.SEND_FILE_OPT(), evt.postStopStream);
}

exports.imageStream = imageStream;

//-----------------------------------------------------------------------------

function thumbStreamHandler(srq) {
  var uid = srq.uid;
  var md  = minfo.get(uid);
  if (!md) {
    log.warn('Thumb media data not in cache: %s', uid);
    srq.res.sendStatus(404);
    return;
  }
  srq.prf = prf.TMB_SHORT;
  htp.setDlnaHeaders(srq, htp.TRANSFER_INTERACTIVE);
  tmb.sendThumb(md, srq.req, srq.res);
}

hnd.addStreamHandler(TMB_HND, thumbStreamHandler);

//-----------------------------------------------------------------------------

function imageStreamHandler(srq) {
  var uid = srq.uid;
  var md = minfo.get(uid);
  if (!md) {
    log.warn('Image media data not in cache: %s', uid);
    srq.res.sendStatus(404);
    return;
  }
  srq.prf = prf.IMG_SHORT;
  var tc = tcodeParams(srq.dev, md);
  if (!tc.tcode) {
    imageStream(srq, rti.fpath(md));
  }
  else {
    htp.setDlnaHeaders(srq, htp.TRANSFER_INTERACTIVE);
    tmb.sendScale(md, tc.w, tc.h, srq.res);
  }
}

hnd.addStreamHandler(IMG_HND, imageStreamHandler);

//-----------------------------------------------------------------------------

function tcodeParams(dev, md) {
  var tcode = false;
  var w = md[id.width];
  var h = md[id.height];
  var r = dev.sreso;
  if ((w < 0 || h < 0) || (downscale && (w > downscale[0] || h > downscale[1]))) {
    tcode = true;
    var wh = alg.scale(w, h, r[0], r[1]);
    w = wh[0];
    h = wh[1];
  }
  if (!tcode && autorotate && md[id.orn] > 4) {
    tcode = true;
  }
  if (!tcode && md[id.fmime] !== mime.jpeg) {
    tcode = true;
  }
  if (!tcode && md[id.bpp] !== 24) {
    tcode = true;
  }
  return { tcode: tcode, w: w, h: h };
}

//-----------------------------------------------------------------------------

function onConfig() {
  autorotate = cfg.imageRotate();
  downscale  = cfg.imageScale();
  if (downscale === 'none') {
    downscale = null;
  }
  else {
    var wh = downscale.split('x');
    downscale = [parseInt(wh[0], 10), parseInt(wh[1], 10)];
  }
}

evt.onConfigReadyChange(onConfig);