/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// DLNA Profiles

'use strict';

var evt = require('../evt');
var rti = require('../rti');
var flg = require('../cls/flags');

var notNull = rti.notNull;

var id = {
  name: 'name',
  mime: 'mime',
  flag: 'flag',
  oper: 'oper',
  conv: 'conv'
};

function Profile(name, mime, flags, operation, conversion) {
  this[id.name] = name;
  this[id.mime] = mime;
  this[id.flag] = flags;
  this[id.oper] = operation;
  this[id.conv] = conversion;
}

Profile.prototype.getName = function() {
  return this[id.name];
};

Profile.prototype.setName = function(name) {
  this[id.name] = name;
};

Profile.prototype.getMime = function() {
  return this[id.mime];
};

Profile.prototype.setMime = function(mime) {
  this[id.mime] = mime;
};

Profile.prototype.getFlags = function() {
  return this[id.flag];
};

Profile.prototype.setFlags = function(flags) {
  this[id.flag] = flags;
};

Profile.prototype.getOper = function() {
  return this[id.oper];
};

Profile.prototype.setOper = function(operation) {
  this[id.oper] = operation;
};

Profile.prototype.getConv = function() {
  return this[id.conv];
};

Profile.prototype.setConv = function(conversion) {
  this[id.conv] = conversion;
};

Profile.prototype.toString = function(short) {
  var rv = (short) ? '' : 'http-get:*:' + this[id.mime] + ':';
  var val = this[id.name];
  if (notNull(val)) {
    rv += 'DLNA.ORG_PN=' + val + ';';
  }
  val = this[id.oper];
  if (notNull(val)) {
    rv += 'DLNA.ORG_OP=' + val + ';';
  }
  val = this[id.conv];
  if (notNull(val)) {
    rv += 'DLNA.ORG_CI=' + val + ';';
  }
  val = this[id.flag];
  if (notNull(val)) {
    rv += 'DLNA.ORG_FLAGS=' + val;
  }
  return rv;
};

exports.Profile = Profile;

exports.newConvProfile = function(name, mime, tseek) {

  var f = (tseek) ? flg.STIME : flg.SNONE;
  var o = (tseek) ? flg.op.timeseek : flg.op.none;

  return new Profile(name, mime, f, o, flg.ci.transcoded);

};

//-----------------------------------------------------------------------------
// Formats
//-----------------------------------------------------------------------------

var fmt = {
  mpegps: 'mpeg-ps',
  mpegts: 'mpeg-ts',
  mpeg4:  'mpeg-4',
  avi:    'avi',
  mkv:    'matroska',
  divx:   'divx',
  qt:     'quicktime',
  flv:    'flash video',
  wmv:    'windows media',
  bdav:   'bdav'
};

exports.fmt = fmt;

//-----------------------------------------------------------------------------
// DLNA Mime
//-----------------------------------------------------------------------------

var mime = {
  jpeg: 'image/jpeg',
  png : 'image/png',
  bmp : 'image/x-ms-bmp',
  mp3 : 'audio/mpeg',
  lpcm: 'audio/L16', // audio/L16;rate=48000;channels=2
  flac: 'audio/flac',
  ogg:  'audio/ogg',
  wma : 'audio/x-ms-wma',
  wav : 'audio/wav',
  adts: 'audio/vnd.dlna.adts',
  mpeg: 'video/mpeg',
  mpts: 'video/vnd.dlna.mpeg-tts',
  wmv : 'video/x-ms-wmv',
  ac3 : 'audio/vnd.dolby.dd-raw',
  dts : 'audio/vnd.dts',
  mp4a: 'audio/mp4',
  mp4v: 'video/mp4',
  mkv:  'video/x-matroska',
  avi:  'video/avi',
  mov:  'video/quicktime',
  srt:  'text/srt'
};

exports.mime = mime;

//-----------------------------------------------------------------------------
// Image
//-----------------------------------------------------------------------------

var img = {
  JPEG_TN:  mime.jpeg,
  JPEG_LRG: mime.jpeg,
  PNG_TN:   mime.png,
  PNG_LRG:  mime.png
};

var JPEG_TN  = new Profile('JPEG_TN', mime.jpeg, flg.PHOTO);
var JPEG_LRG = new Profile('JPEG_LRG', mime.jpeg, flg.PHOTO);

exports.IMG_SHORT = JPEG_LRG.toString(true);
exports.IMG_FULL  = JPEG_LRG.toString();
exports.TMB_SHORT = JPEG_TN.toString(true);
exports.TMB_FULL  = JPEG_TN.toString();

exports.image = function(name) {
  return new Profile(name, img[name]);
};

//-----------------------------------------------------------------------------
// Audio
//-----------------------------------------------------------------------------

var aud = {
  MP3:           mime.mp3,
  LPCM:          mime.lpcm,
  LPCM_MPS:      mime.lpcm,
  WMABASE:       mime.wma,
  WMAFULL:       mime.wma,
  WMAPRO:        mime.wma,
  AAC_ISO:       mime.mp4a,
  AAC_MULT5_ISO: mime.mp4a,
  AC3:           mime.ac3,
  DTS:           mime.dts
};

exports.audio = function(name) {
  return new Profile(name, aud[name]);
};

//-----------------------------------------------------------------------------
// Video
//-----------------------------------------------------------------------------

var vid = {

  MPEG_PS_PAL:       mime.mpeg,
  MPEG_PS_NTSC:      mime.mpeg,

  MPEG_TS_SD_EU:     mime.mpts, // Transport Packet with zero value timestamp
  MPEG_TS_SD_EU_T:   mime.mpts, // Transport Packet with a valid non-zero timestamp
  MPEG_TS_SD_EU_ISO: mime.mpeg, // Transport Packet without a Timestamp field

  MPEG_TS_SD_NA:     mime.mpts,
  MPEG_TS_SD_NA_T:   mime.mpts,
  MPEG_TS_SD_NA_ISO: mime.mpeg,

  MPEG_TS_HD_NA:     mime.mpts,
  MPEG_TS_HD_NA_T:   mime.mpts,
  MPEG_TS_HD_NA_ISO: mime.mpeg,

  WMVMED_BASE:       mime.wmv,
  WMVMED_FULL:       mime.wmv,
  WMVMED_PRO:        mime.wmv,
  WMVHIGH_FULL:      mime.wmv,
  WMVHIGH_PRO:       mime.wmv,

  AVC_MKV_MP_HD_AC3:       mime.mkv,
  AVC_MKV_MP_HD_AAC_MULT5: mime.mkv,
  AVC_MKV_HP_HD_AC3:       mime.mkv,
  AVC_MKV_HP_HD_AAC_MULT5: mime.mkv,

  AVC_MP4_MP_SD:  mime.mp4v,
  AVC_MP4_HP_HD:  mime.mp4v

};

exports.video = function(name) {
  return new Profile(name, vid[name]);
};

//-----------------------------------------------------------------------------
// TV System
//-----------------------------------------------------------------------------

exports.PAL  = 0;
exports.NTSC = 1;

exports.ifPAL = function(tvs, pal, ntsc) {
  return (tvs === 1) ? ntsc : pal;
};

//-----------------------------------------------------------------------------

var all = [];

//-----------------------------------------------------------------------------

function onDlnaSource(sourceArray) {
  for (var i = 0; i < all.length; i++) {
    sourceArray.push(all[i]);
  }
}

evt.onDlnaSource(onDlnaSource);

//-----------------------------------------------------------------------------

function initAll() {

  function add(map, flags) {
    for (var key in map) {
      var prf = new Profile(key, map[key], flags);
      all.push(prf.toString());
    }
  }

  add(img, flg.PHOTO);
  add(aud, flg.MUSIC);
  add(vid, flg.VIDEO);

}

initAll();

//-----------------------------------------------------------------------------
// Mime Extensions
//-----------------------------------------------------------------------------

var xmime = require('mime');

xmime.define({
  'video/mp2t':              ['m2ts', 'm2t', 'ts'],
  'video/avi':               ['avi'],
  'video/mp4':               ['m4v', 'mp4'],
  'audio/vnd.dolby.dd-raw':  ['ac3'],
  'text/srt':                ['srt']
});
