/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var path = require('path');
var fs   = require('fs');

var evt = require('../evt');
var log = require('../log');
var agt = require('../agt');
var cfg = require('../cfg');
var rti = require('../rti');
var htp = require('../htp');
var mdb = require('./mdb');

var NO_CACHE_PARAM = 'nocache';

var id = rti.id;
var thumb = mdb.thumb;
var minfo = mdb.minfo;
var scale = mdb.scale;

//-----------------------------------------------------------------------------

function sendThumb(md, req, res) {

  var uid = rti.uid(md);

  var tpath = thumb.filePath(uid);

  var nocache = req.query[NO_CACHE_PARAM];

  if (nocache === undefined && thumb.has(uid)) {
    res.sendFile(tpath, htp.SEND_FILE_OPT());
    return;
  }

  var ftype = md[id.ftype];

  if (ftype === rti.ftype.unknown) {
    res.status(404).send('Unknown file type');
    return;
  }

  var spath  = rti.fpath(md);
  var mtype  = md[id.mtype];
  var index  = md[id.index] || 0;
  var width  = md[id.twidth];
  var height = md[id.theight];
  var orn    = md[id.orn] || 0;
  var bcolor = cfg.thumbColor();

  agt.scale(spath, tpath, ftype, mtype, index, width, height, bcolor, orn, function(err, size) {

    if (err) {
      log.error('Thumbnail generation error: %s', err);
      md[id.tvalid] = false;
      minfo.set(uid, md);
      res.status(500).send(err.toString());
      return;
    }

    res.sendFile(tpath, htp.SEND_FILE_OPT());

    if (rti.isNumber(size)) {
      thumb.set(uid, size);
    }
    else {
      log.warn('Thumb response is not a number: %s', typeof size);
    }

  });


}

exports.sendThumb = sendThumb;

//-----------------------------------------------------------------------------

function sendScale(md, width, height, res) {

  var uid = rti.uid(md);

  uid += '@' + width + 'x' + height;

  var tpath = scale.filePath(uid);

  if (scale.has(uid)) {
    evt.postStartStream();
    res.sendFile(tpath, htp.SEND_FILE_OPT(), evt.postStopStream);
    return;
  }

  var ftype = md[id.ftype];

  if (ftype === rti.ftype.unknown) {
    res.status(404).send('Unknown file type');
    return;
  }

  var spath  = rti.fpath(md);
  var mtype  = md[id.mtype];
  var index  = md[id.index] || 0;
  var orn    = md[id.orn] || 0;
  var bcolor = -1;

  agt.scale(spath, tpath, ftype, mtype, index, width, height, bcolor, orn, function(err, size) {

    if (err) {
      log.error('Scale image generation error: %s', err);
      res.status(500).send(err.toString());
      return;
    }

    evt.postStartStream();
    res.sendFile(tpath, htp.SEND_FILE_OPT(), evt.postStopStream);

    if (rti.isNumber(size)) {
      scale.set(uid, size);
    }
    else {
      log.warn('Scale response is not a number: %s', typeof size);
    }

  });

}

exports.sendScale = sendScale;

//-----------------------------------------------------------------------------

function serveThumb(req, res) {

  var uid = req.params[0];

  if (!uid || uid.length < 32) {
    res.status(400).send('Invalid thumb UID');
    return;
  }

  var md = minfo.get(uid);

  if (!md) {
    log.error('Media info not in cache for thumb: %s', uid);
    res.status(404).send('Media info not found for thumbnail generation');
    return;
  }

  sendThumb(md, req, res);

}

evt.onHttpAppReady(function(app) {
  cfg.dlna(app).get('/thumb/*.jpg', serveThumb);
});