/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */


'use strict';

var evt = require('../evt');
var env = require('../env');
var hnd = require('../hnd');
var log = require('../log');
var rti = require('../rti');
var htp = require('../htp');
var cfg = require('../cfg');
var jsl = require('../jsl');
var mdb = require('./mdb');
var tmb = require('./tmb');
var img = require('./img');
var alg = require('./alg');
var prf = require('./prf');
var cdr = require('../cdr');
var didl = require('../cls/didl');
var flag = require('../cls/flags');
var ffm  = require('../ffm');
var base = require('../menc/menc');

var id    = rti.id;
var mtype = rti.mtype;
var pmime = prf.mime;

var minfo = mdb.minfo;

var VID_HND = 'vid';
var MOV_HND = 'mov';

var vx = {
  enableText: null,
  textLang:   null,
  audioLang:  null,
  audioPlay:  null,
  textPlay:   null,
  trackShow:  null
};

var vv = {
  autoPlay:  0,
  handPlay:  1,
  autoFirst: 2,
  showDir: 0,
  showNone: 100
};

//-----------------------------------------------------------------------------
// Media Handler
//-----------------------------------------------------------------------------

function videItem(brq, md, cb) {

  if (brq.mt !== mtype.video || !rti.isVideo(md)) return;

  if (!rti.getSubFlag(md)) {
    rti.setSubFlag(md);
    var submap = brq.text;
    var fname = md[id.fname] || '';
    fname = fname.toLowerCase();
    for (var key in submap) {
      if (jsl.hasPrefix(key, fname)) {
        log.debug('Subfile found: %s', key);
        rti.getSubTracks(md).push(submap[key]);
      }
    }
    minfo.upd(rti.uid(md), md);
  }

  var cnt = getMultiTrackCount(brq.dev, md);

  if (cnt > 1) {
    cb(null, makeVideoFolder(brq, md, cnt));
  }
  else {
    cb(null, makeVideoItem(brq, md));
  }

  return true;

}

hnd.addMediaHandler(videItem);

//-----------------------------------------------------------------------------
// Video Folder
//-----------------------------------------------------------------------------

function makeVideoFolder(brq, md, cnt) {

  var dir = new didl.Container(hnd.mkoid(MOV_HND, rti.uid(md)), brq.oid, rti.fname(md, true), cnt);

  dir.setDate(md[id.mtime]);

  dir.setMultiItem(true);

  return dir;

}


//-----------------------------------------------------------------------------
// Video Item
//-----------------------------------------------------------------------------

function makeVideoItem(brq, md, an, sn) {

  var track = '';

  if (isTrackNum(an)) {
    var aud = rti.audioTrack(md, an);
    var lng = rti.audioLangName(aud, String(an + 1));
    track = 'Audio ' + lng.toUpperCase();
  }

  if (isTrackNum(sn)) {
    var sub = rti.textTrack(md, sn);
    var slg = rti.textLangName(sub, String(sn + 1));
    if (track.length > 0) track += ', ';
    track += 'Text ' + slg.toUpperCase();
  }

  if (track.length > 0) {
    track = '[' + track + '] ';
  }

  var title = track + rti.fname(md, cfg.hideExt());

  var iid = hnd.mkoid(VID_HND, viduid(md, an, sn));

  var vid = didl.videoItem(iid, brq.oid, title);

  vid.setDate(md[id.mtime]);

  vid.addResource(videoRes(brq, md, an, sn));

  if (brq.dev.vtmb) {
    vid.addResource(img.thumbRes(brq, md));
  }

  return vid;

}

function viduid(md, an, sn) {
  var uid = rti.uid(md);
  if (isTrackNum(an)) uid += '-' + an;
  if (isTrackNum(sn)) uid += '-' + sn;
  return uid;
}

//-----------------------------------------------------------------------------

function videoRes(brq, md, an, sn) {

  var uid = viduid(md, an, sn);

  var url = hnd.streamURL(brq.surl, VID_HND, uid);

  var opt = getOptions(brq.dev, md, an, sn);

  var pro = opt.pro;

  var res = new didl.Res(url, pro);

  res.setDuration(md[id.duration]);

  res.setBitrate(md[id.bitrate]);

  if (!pro.getConv()) {
    res.setSize(md[id.fsize]);
  }

  return res;

}

//-----------------------------------------------------------------------------

function getMultiTrackCount(dev, md) {

  if (!isTranscode(dev, md) || vx.trackShow === vv.showNone) {
    return 0;
  }

  var ac = rti.audioCount(md);

  var mc = 0;

  if (ac === 1) {
    mc = 1;
  }
  else if (ac > 1) {
    switch (vx.audioPlay) {
      case vv.handPlay:
        mc = ac;
        break;
      case vv.autoPlay:
        if (rti.audioLangIndex(md, vx.audioLang) === -1) {
          mc = ac;
        }
        else {
          mc = 1;
        }
        break;
      case vv.autoFirst:
        mc = 1;
        break;
    }
  }

  if (vx.enableText) {
    var sc = rti.textCount(md);
    if (sc > 0) {
      ac = Math.max(mc, 1);
      switch (vx.textPlay) {
        case vv.handPlay:
          mc += sc * ac;
          break;
        case vv.autoPlay:
          if (rti.textLangIndex(md, vx.textLang) === -1) {
            mc += sc * ac;
          }
          break;
      }
    }
  }

  return mc;

}

//-----------------------------------------------------------------------------

function isTranscode(dev, md) {
  return (dev.vcode !== 'none' && !base.isNative(dev, md));
}

function isTrackNum(val) {
  return (typeof val === 'number');
}

//-----------------------------------------------------------------------------
// Movide Folder Browse
//-----------------------------------------------------------------------------

function movBrowse(brq, cb) {

  var uid = brq.uid;
  var md  = minfo.get(uid);

  if (!md) {
    return cb('Video file is not in media cache: ' + brq.oid);
  }

  var cnt = getMultiTrackCount(brq.dev, md);

  if (brq.meta) {
    brq.append(makeVideoFolder(brq, md, cnt));
    return cb();
  }

  brq.total = cnt;

  var ac = rti.audioCount(md);
  var ax = -1;

  if (ac > 0 && vx.audioPlay !== vv.handPlay) {
    ax = rti.audioLangIndex(md, vv.audioLang);
    if (ax < 0 && vx.audioPlay === vv.autoFirst) {
      ax = 0;
    }
  }

  if (ax < 0) {
    for (var i = 0; i < ac; i++) {
      brq.append(makeVideoItem(brq, md, i, null));
    }
  }
  else {
    brq.append(makeVideoItem(brq, md, ax, null));
  }

  if (vx.enableText) {
    var sc = rti.textCount(md);
    if (sc > 0) {
      if (ax < 0) {
        for (i = 0; i < ac; i++) {
          for (var t = 0; t < sc; t++) {
            brq.append(makeVideoItem(brq, md, i, t));
          }
        }
      }
      else {
        for (t = 0; t < sc; t++) {
          brq.append(makeVideoItem(brq, md, ax, t));
        }
      }
    }
  }

  cb();

}

hnd.addBrowseHandler(MOV_HND, movBrowse);

//-----------------------------------------------------------------------------
// Video Item Browse
//-----------------------------------------------------------------------------

function vidBrowse(brq, cb) {

  var ary = brq.uid.split('-');
  var len = ary.length;
  var uid = ary[0];
  var md  = minfo.get(uid);
  var an  = (len > 1) ? ary[1] : null;
  var sn  = (len > 2) ? ary[2] : null;

  if (!md) {
    return cb('Video file is not in media cache: ' + brq.oid);
  }

  if (brq.meta) {
    brq.append(makeVideoItem(brq, md, an, sn));
  }

  cb();

}

hnd.addBrowseHandler(VID_HND, vidBrowse);

//-----------------------------------------------------------------------------
// Video Stream Handlers
//-----------------------------------------------------------------------------

function videoStreamHandler(srq) {

  var ary = srq.uid.split('-');
  var len = ary.length;
  var uid = ary[0];
  var an  = (len > 1) ? parseInt(ary[1], 10) : null;
  var sn  = (len > 2) ? parseInt(ary[2], 10) : null;
  var req = srq.req;
  var res = srq.res;
  var md  = minfo.get(uid);

  if (!md) {
    log.warn('Video media data not in cache: %s', uid);
    res.sendStatus(404);
    return;
  }

  var opt = getOptions(srq.dev, md, an, sn);
  var pro = opt.pro;

  srq.prf = pro.toString(true);

  htp.setContentType(res, pro.getMime());

  htp.setDlnaHeaders(srq, htp.TRANSFER_STREAMING);

  if (!pro.getConv()) {
    evt.postStartStream(req.ip);
    res.sendFile(rti.fpath(md), htp.SEND_FILE_OPT(), evt.stopStreamCb(req.ip));
    return;
  }

  // Transcoding

  var time = htp.getTimeRange(req, opt.len);

  htp.setTimeRange(res, srq.dev, time);

  res.set('Connection', 'close');
  res.set('Transfer-Encoding', 'chunked');

  if (time) {
    res.status(206);
  }

  if (req.method === 'HEAD') {
    res.end();
    return;
  }

  opt.res = res;
  opt.ecb = evt.stopStreamCb(req.ip);
  opt.pos = (time) ? time.start : 0;
  opt.url = rti.fpath(md);

  evt.postStartStream(req.ip);

  cdr.run(opt);

}

hnd.addStreamHandler(VID_HND, videoStreamHandler);

//-----------------------------------------------------------------------------
// Video Profiles
//-----------------------------------------------------------------------------

function getOptions(dev, md, an, sn) {

  var opt = new ffm.Options();

  var vcode = dev.vcode;

  if (!isTranscode(dev, md)) {
    opt.pro = getNativeProfile(dev, md);
    return opt;
  }

  if (!isTrackNum(an)) {
    an = rti.audioLangIndex(md, vx.audioLang);
    if (an === -1) an = 0;
  }

  if (!isTrackNum(sn)) {
    sn = -1;
    if (vx.enableText && vx.textPlay !== vv.handPlay) {
      var sc = rti.textCount(md);
      if (sc > 0) {
        sn = rti.textLangIndex(md, vx.textLang);
        if (sn < 0 && vx.textPlay === vv.autoFirst) sn = 0;
      }
    }
  }

  opt.len = rti.duration(md);
  opt.vid = rti.videoTrack(md);
  opt.fps = rti.frameRate(md);
  opt.aud = (an > -1) ? rti.audioTrack(md, an) : null;
  opt.sub = (sn > -1) ? rti.textTrack(md, sn) : null;

  hnd.getProfile(vcode, dev, opt);

  if (!opt.pro) {
    log.warn('Video transcoding not found for: %s (%s)', vcode, rti.fpath(md));
    opt.pro = getNativeProfile(dev, md);
  }

  return opt;

}

//-----------------------------------------------------------------------------

function getNativeProfile(dev, md) {

  var fmime  = rti.fmime(md);
  var fcodec = rti.codec(md);
  var atrack = (rti.audioCount(md) > 0) ? rti.audioTrack(md, 0) : null;
  var achann = (atrack) ? atrack[id.channels] : 6;
  var pname  = null;

  var fmt = prf.fmt;
  var tvs = dev.tvs;

  switch (fcodec) {
    case fmt.mpegps:
      pname = prf.ifPAL(tvs, 'MPEG_PS_PAL', 'MPEG_PS_NTSC');
      break;
    case fmt.mpegts:
    case fmt.bdav:
      pname = prf.ifPAL(tvs, 'MPEG_TS_SD_EU', 'MPEG_TS_HD_NA');
      break;
    case fmt.wmv:
      pname = (achann < 5) ? 'WMVMED_FULL' : 'WMVMED_PRO';
      break;
    case fmt.avi:
      pname = 'AVI';
      break;
    case fmt.divx:
      pname = 'DIVX';
      break;
    case fmt.flv:
      pname = 'FLV';
      break;
    case fmt.mkv:
      pname = 'AVC_MKV_MP_HD_AC3';
      break;
    case fmt.mpeg4:
      pname = 'AVC_MP4_MP_SD';
      break;
  }

  fmime = dev.mime[rti.codec(md)] || fmime;

  return new prf.Profile(pname, fmime, flag.SBYTE, flag.op.range, flag.ci.none);

}

//-----------------------------------------------------------------------------

function updateConfig() {
  var cobj = cfg.getConfig();
  vx.enableText = cobj.enableText;
  vx.audioLang  = cobj.audioLang;
  vx.audioPlay  = cobj.audioPlay;
  vx.textLang   = cobj.textLang;
  vx.textPlay   = cobj.textPlay;
  vx.trackShow  = cobj.trackShow;
  cobj = null;
}

evt.onConfigReadyChange(updateConfig);