/*
 * Copyright (c) 2015 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var env = require('./env');
var log = require('./log');
var cfg = require('./cfg');
var rti = require('./rti');
var jsl = require('./jsl');
var txt = require('./dlna/txt');

exports.mux = {
  aiff:  'aiff',
  raw:   'null',
  vob:   'vob',
  ts:    'mpegts',
  tsraw: 'mpegtsraw'
};

exports.codec = {
  mp2v:  'mpeg2video',
  h264:  'h264',
  copy:  'copy',
  pcm:   'pcm_s16be',
  mp2a:  'mp2',
  ac3:   'ac3'
};

var fid = {
  hwaccel: '-hwaccel'
};

var VC_COPY = 'copy';

//var PIC_SUB = ['rle', 's_vobsub', 'dxsb'];

//-----------------------------------------------------------------------------

function Options() {

  this.bin = null; // Encoder path

  this.res = null; // Response
  this.ecb = null; // End callback

  this.vid = null; // Video track
  this.aud = null; // Audio track
  this.sub = null; // Subtitles track or file path

  this.len = 0;    // Media duration, needed for profile building
  this.pos = 0;    // Start time
  this.fps = 0;    // Video frame rate

  this.pro = null; // Profile

  this.url = null; // Encoding source
  this.inp = null; // User input options

  // Video

  this.vcc = null; // Video codec
  this.vbr = null; // Video bitrate
  this.mbr = null; // Video max bitrate
  this.buf = null; // Buffer size
  this.enc = null; // User encoder options

  // Audio
  this.acc = null; // Audio codec
  this.abr = null; // Audio bitrate
  this.chn = null; // Channels
  this.asr = null; // Audio sample rate

  this.mux = null; // Muxer

}

//-----------------------------------------------------------------------------

Options.prototype.buildArgs = function(cb) {

  var opt = this;
  var sub = opt.sub;

  if (sub && !rti.fpath(sub) && !isPicSub(sub)) {
    txt.getSrtPath(opt.url, rti.streamIndex(sub), function(err, srtPath) {
      if (err) {
        opt.sub = null;
      }
      cb(null, build(opt, srtPath));
    });
    return;
  }

  cb(null, build(opt, null));

};

function build(opt, srtPath) {

  var arg = [];

  // Encoder path

  if (!opt.bin) {
    opt.bin = cfg.getEncBin(env.coderPath);
  }

  // Input

  if (opt.pos > 0) {
    var ss = Math.round(opt.pos / 1000);
    arg.push('-ss', ss);
  }

  if (opt.vid) {
    jsl.aryAddStr(arg, opt.inp, ' ');
//    if (cfg.getVideoHW() && !hasOpt(arg, fid.hwaccel)) {
//      arg.push(fid.hwaccel, 'auto');
//    }
  }

  arg.push('-i', opt.url);

  // Output

  if (opt.vid) {
    setVideoOptions(arg, opt, opt.vid, srtPath);
    jsl.aryAddStr(arg, opt.enc, ' ');
  }
  else {
    arg.push('-vn');
  }

  if (opt.aud) {
    setAudioOptions(arg, opt, opt.aud);
  }
  else {
    arg.push('-an');
  }

  arg.push('-sn');

  // Filters

  // Muxer

  arg.push('-f', opt.mux);

  arg.push('pipe:1');

  return arg;

}

exports.Options = Options;

//-----------------------------------------------------------------------------

function setStreamMap(arg, type, stream) {
  var ix = rti.streamIndex(stream);
  arg.push('-map', '0:' + type + ':' + ix);
}

//-----------------------------------------------------------------------------

function setVideoOptions(arg, opt, vid, srtPath) {

  var sub = opt.sub;
  var sfp = null;
  var pic = false;
  var complex = false;
  var aspect  = false;

  // Subtitles Init

  if (sub) {
    sfp = rti.fpath(sub);
    if (sfp) {
      sub = rti.textTrack(sub, 0);
    }
    pic = isPicSub(sub);
  }

  // Video Filter Graph

  var vf = [];

  if (checkDeinterlace(vid)) {
    vf.push('w3fdif');
  }

  if (rti.getDSR(vid) > 1.8 && cfg.getVideoPad()) {
    vf.push('scale=iw*sar:ih');
    vf.push('pad=max(iw\\,ih*(16/9)):ow/(16/9):(ow-iw)/2:(oh-ih)/2');
    aspect = true;
  }

  // Picture Subtitles
  // [0:v:0][0:s:0]overlay=0:H-h[vid]
  // ffmpeg -i input.avi  -vf "[in]crop=in_w:in_h-20:0:0 [crop]; [crop]scale=320:240 [scale];[scale]subtitles=srt.srt"

  if (pic) {
    var vsx = 0;
    if (sfp) {
      arg.push('-i', sfp);
      vsx = 1;
    }
    var fx = '[0:v]';
    if (vf.length > 0) {
      fx += vf.join(',');
      fx += '[scaled];[scaled]';
    }
    fx += '[' + vsx + ':s:' + rti.streamIndex(sub) + ']overlay[out]';
    arg.push('-filter_complex', fx);
    arg.push('-map', '[out]');
    complex = true;
  }
  else {
    setStreamMap(arg, 'v', vid);
  }

  // Encoder Options

  //arg.push('-copyts');

  addArg(arg, '-vcodec', opt.vcc);

  if (opt.vcc === VC_COPY) return;

  if (opt.mbr > -1) {
    addArg(arg, '-bufsize', opt.buf, 'k');
    addArg(arg, '-maxrate', opt.vbr + opt.mbr, 'k');
  }

  addArg(arg, '-vb', opt.vbr, 'k');

  if (opt.fps < 20) {
    arg.push('-r', 25);
  }

  // Text Subtitles

  if (sub && !pic) {
    arg.push('-copyts');
    arg.push('-start_at_zero');
    if (!srtPath) srtPath = sfp;
    if (jsl.hasSuffix(srtPath, 'ass')) {
      vf.push('ass=' + filterEscape(srtPath));
    }
    else {
      var fsub = 'subtitles=f=';
      fsub += filterEscape(srtPath);
      fsub += ':charenc=' + cfg.getTextCharset();
      vf.push(fsub);
    }
  }

  if (!complex && vf.length > 0) {
    arg.push('-vf', vf.join(','));
  }

  if (aspect) {
    arg.push('-aspect', '16:9');
  }

}

//-----------------------------------------------------------------------------

function checkDeinterlace(vid) {
  return (cfg.getDeinterlace() && rti.isInterlaced(vid) && rti.width(vid) < 1280);
}

//-----------------------------------------------------------------------------

function setAudioOptions(arg, opt, aud) {
  setStreamMap(arg, 'a', aud);
  addArg(arg, '-acodec', opt.acc);
  addArg(arg, '-ab', opt.abr);
  addArg(arg, '-ac', opt.chn);
  addArg(arg, '-ar', opt.asr);
  if (opt.vid) {
    arg.push('-af', 'aresample=async=1000')
  }
}

//-----------------------------------------------------------------------------
// Utilities
//-----------------------------------------------------------------------------

function filterEscape(path) {
  path = path.replace(/\\/g, '\\\\');
  path = path.replace(/'/g, "\\'");
  path = path.replace(/,/g, "\\,");
  path = path.replace(/;/g, "\\;");
  path = path.replace(/\[/g, "\\[");
  path = path.replace(/\]/g, "\\]");
  path = path.replace(/:/, '\\:');
  return "'" + path + "'";
}

exports.filterEscape = filterEscape;

//-----------------------------------------------------------------------------

function addArg(args, name, value, suffix) {
  if (value !== null && value !== undefined) {
    if (suffix !== undefined && suffix !== null) {
      value += suffix;
    }
    args.push(name, String(value));
  }
}

function hasOpt(args, name) {
  var len = args.length;
  for (var i = 0; i < len; i++) {
    if (args[i] === name) {
      return true;
    }
  }
  return false;
}

function isPicSub(sub) {

  var cid = rti.codecid(sub);

  switch (cid) {
    case 'rle':
    case 's_vobsub':
    case 'dxsb':
      return true;
  }

  return false;

}