/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// Media Handler Hooks

'use strict';

var async = require('async');
var log = require('./log');
var dev = require('./ssdp/dev');
var StreamRequest = require('./cls/hrequest').StreamRequest;

var ROOT  = '0';
var PHOTO = '16';
var MUSIC = '7';
var VIDEO = '15';
var FILE  = 'fs';

exports.ROOT  = ROOT;
exports.PHOTO = PHOTO;
exports.MUSIC = MUSIC;
exports.VIDEO = VIDEO;
exports.FILE  = FILE;

var hbrowse = {}; // Browsing handlers
var hstream = {}; // HTTP stream handlers
var hmedia  = []; // Media data handlers
var hcount  = {}; // Count Handlers
var tcoder  = {}; // Transcoder options builders

//-----------------------------------------------------------------------------
// Browse Handlers
//-----------------------------------------------------------------------------

function addBrowseHandler(name, proc) {
  addMapProc(hbrowse, name, proc);
}

exports.addBrowseHandler = addBrowseHandler;

//-----------------------------------------------------------------------------

exports.addRootBrowser = function(proc) {
  addBrowseHandler(ROOT, proc);
};

exports.addPhotoBrowser = function(proc) {
  addBrowseHandler(PHOTO, proc);
};

exports.addMusicBrowser = function(proc) {
  addBrowseHandler(MUSIC, proc);
};

exports.addVideoBrowser = function(proc) {
  addBrowseHandler(VIDEO, proc);
};

exports.addFileBrowser = function(proc) {
  addBrowseHandler(FILE, proc);
};

//-----------------------------------------------------------------------------

function browseRequest(brq, cb) {

  var oid = brq['oid'];
  var pos = oid.indexOf('_');
  var hname;

  if (pos === -1) {
    hname = oid;
  }
  else {
    hname = oid.substring(0, pos);
    brq['uid'] = oid.substring(pos + 1);
  }

  runBrowseHandlers(hname, brq, true, cb);

}

exports.browseRequest = browseRequest;

//-----------------------------------------------------------------------------

function runBrowseHandlers(handlerName, brq, errorIfNotFound, cb) {

  var ary = hbrowse[handlerName];

  if (!ary) {
    if (errorIfNotFound) {
      cb('Handler not found for oid: ' + brq.oid);
    }
    else {
      cb();
    }
  }
  else if (typeof ary === 'function') {
    ary(brq, cb);
  }
  else {
    async.applyEachSeries(ary, brq, cb);
  }

}

exports.runBrowseHandlers = runBrowseHandlers;

//-----------------------------------------------------------------------------

exports.mkoid = function(handle, uid) {
  return handle + '_' + uid;
};

exports.fsoid = function(mediaType, uid) {
  return FILE + '_' + mediaType + uid;
};

exports.parseFsoid = function(uid) {
  return {
    mt:  parseInt(uid.charAt(0), 10),
    uid: uid.substring(1)
  };
};

//-----------------------------------------------------------------------------
// Stream Handler
//-----------------------------------------------------------------------------

function addStreamHandler(name, proc) {
  if (hstream[name] !== undefined) {
    throw new Error('Stream handler already exists: ' + name);
  }
  hstream[name] = proc;
}

exports.addStreamHandler = addStreamHandler;

//-----------------------------------------------------------------------------

function onStreamRequest(req, res) {

  var hnd = req.param('hnd');
  var uid = req.param('uid');

  var proc = hstream[hnd];

  if (!proc) {
    log.error('Stream handler not found: %s', req.originalUrl);
    res.sendStatus(404);
  }
  else {
    res.setTimeout(30 * 60 * 1000); // 30 minutes
    var rdv = dev.getDeviceConfig(req.ip);
    var srq = new StreamRequest(rdv, req, res, uid);
    proc(srq);
  }

}

//-----------------------------------------------------------------------------

function initStreamRouter(route, path) {
  route.use(path + '/:hnd/:uid', onStreamRequest);
}

exports.initStreamRouter = initStreamRouter;

exports.streamURL = function(url, hnd, uid) {
  return url + '/' + hnd + '/' + uid;
};

//-----------------------------------------------------------------------------
// Media data (convert md to didl)
//-----------------------------------------------------------------------------

function addMediaHandler(proc) {
  hmedia.unshift(proc);
}

exports.addMediaHandler = addMediaHandler;

//-----------------------------------------------------------------------------

function processMedia(brq, md, cb) {
  var len = hmedia.length;
  for (var i = 0; i < len; i++) {
    if (hmedia[i](brq, md, cb) === true) return;
  }
  cb(null, null);
}

exports.processMedia = processMedia;

//-----------------------------------------------------------------------------
// Top Count Handlers
//-----------------------------------------------------------------------------

function addCountHandler(name, proc) {
  addMapProc(hcount, name, proc);
}

//-----------------------------------------------------------------------------

exports.addRootCount = function(proc) {
  addCountHandler(ROOT, proc);
};

exports.addPhotoCount = function(proc) {
  addCountHandler(PHOTO, proc);
};

exports.addMusicCount = function(proc) {
  addCountHandler(MUSIC, proc);
};

exports.addVideoCount = function(proc) {
  addCountHandler(VIDEO, proc);
};

exports.count = function(brq, name) {

  var ary = hcount[name];

  if (!ary) return 0;

  if (typeof ary === 'function') {
    return ary(brq);
  }

  var cnt = 0;

  for (var i = 0; i < ary.length; i++) {
    cnt += ary[i](brq);
  }

  return cnt;

};

//-----------------------------------------------------------------------------
// Transcoders
//-----------------------------------------------------------------------------

exports.addTcoder = function(name, proc) {
  tcoder[name] = proc;
};

exports.getProfile = function(name, dev, opt) {
  var coder = tcoder[name];
  return (coder !== undefined) ? coder(dev, opt) : null;
};

//-----------------------------------------------------------------------------
// Helpers
//-----------------------------------------------------------------------------

function addMapProc(map, name, proc) {

  var ary = map[name];

  if (ary === undefined) {
    map[name] = proc;
  }
  else if (typeof ary === 'function') {
    map[name] = [ary, proc];
  }
  else {
    ary.push(proc);
  }

}


