/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var prf = require('../dlna/prf');
var hnd = require('../hnd');
var evt = require('../evt');
var ffm = require('../ffm');
var rti = require('../rti');
var jsl = require('../jsl');
var enc = require('./menc');

var id = rti.id;

//-----------------------------------------------------------------------------

var CNAME = 'mp2v';

var MP2V = {
  pro: 'auto', // Profile
  vbr: 8000,   // Video bit rate in kbits
  mbr: -1,      // Max bit rate as delta to vbr
  buf: 1,      // Buffer size multiplier to vbr
  inp: null,   // Input options
  enc: null,   // Encoding options
  ver: 202     // Cache version
};

exports.DEFS = MP2V;

//-----------------------------------------------------------------------------

var MPEG_PS_PAL       = 'MPEG_PS_PAL';
var MPEG_PS_NTSC      = 'MPEG_PS_NTSC';
var MPEG_TS_SD_EU     = 'MPEG_TS_SD_EU';
var MPEG_TS_SD_NA     = 'MPEG_TS_SD_NA';
var MPEG_TS_SD_EU_ISO = 'MPEG_TS_SD_EU_ISO';
var MPEG_TS_SD_NA_ISO = 'MPEG_TS_SD_NA_ISO';
var MPEG_TS_SD_EU_T   = 'MPEG_TS_SD_EU_T';
var MPEG_TS_SD_NA_T   = 'MPEG_TS_SD_NA_T';

var EU = MPEG_PS_PAL;
var NA = MPEG_PS_NTSC;

//-----------------------------------------------------------------------------

function setAudio(dev, opt) {

  var aud = opt.aud;

  if (!aud) {
    return;
  }
  // Audio Sample Rate

  var asr = enc.audioSampleRate(dev, aud);

  // Audio Channels

  var chn  = aud[id.channels] || 2;
  var abr  = aud[id.bitrate];

  if (chn > dev.chn) {
    chn = dev.chn;
    abr = null;
  }

  if (!abr) {
    abr = enc.audioBitRate(chn);
  }

  opt.acc = ffm.codec.ac3;
  opt.chn = chn;
  opt.asr = asr;
  opt.abr = abr;

}

//-----------------------------------------------------------------------------
// -copyts -bsf h264_mp4toannexb -mpegts_m2ts_mode 1
// -profile main -preset superfast -x264opts keyint=20:min-keyint=10
// -vsync 2 -profile main -preset superfast -x264opts cabac=0:bframes=0:keyint=30:min-keyint=30:scenecut=0
//-----------------------------------------------------------------------------

function x264(dev, opt) {

  // Profile

  var pmime = prf.mime.mpts;

  var tcfg = dev[CNAME] || MP2V;

  var pname = tcfg.pro || MP2V.pro;

  if (pname === 'auto') {
    pname = prf.ifPAL(dev.tvs, MPEG_TS_SD_EU, MPEG_TS_SD_NA);
  }

  opt.pro = prf.newConvProfile(pname, pmime, opt.len);

  // Video

  var eopt = tcfg.enc || '';

  eopt += ' -copyts -mpegts_m2ts_mode 1';

  opt.vcc = ffm.codec.h264;
  opt.vbr = tcfg.vbr || MP2V.vbr;
  opt.mbr = tcfg.mbr || MP2V.mbr;
  opt.buf = tcfg.buf || MP2V.buf;
  opt.inp = tcfg.inp;
  opt.enc = eopt;

  // Audio

  setAudio(dev, opt);

  opt.mux = ffm.mux.ts;

}

//-----------------------------------------------------------------------------
// "scale=iw*sar:ih,pad=max(iw\,ih*(16/9)):ow/(16/9):(ow-iw)/2:(oh-ih)/2"

function mp2v(dev, opt) {

  // Profile

  var pmime = prf.mime.mpeg;

  var tcfg = dev[CNAME] || MP2V;

  var pname = tcfg.pro || MP2V.pro;

  if (pname === 'auto') {
    pname = prf.ifPAL(dev.tvs, EU, NA);
  }

  opt.pro = prf.newConvProfile(pname, pmime, opt.len);

  // Video

  opt.vcc = ffm.codec.mp2v;
  opt.vbr = tcfg.vbr || MP2V.vbr;
  opt.mbr = tcfg.mbr;
  opt.buf = tcfg.buf || MP2V.buf;
  opt.buf = opt.buf * opt.vbr;
  opt.inp = tcfg.inp;
  opt.enc = tcfg.enc;

  // Audio

  setAudio(dev, opt);

  opt.mux = ffm.mux.vob;

}

//-----------------------------------------------------------------------------
// Handler Hooks
//-----------------------------------------------------------------------------


hnd.addTcoder(CNAME, mp2v);

//-----------------------------------------------------------------------------

function onDevDefaults(dev) {
  var opt = dev[CNAME];
  if (!opt || !opt.ver || opt.ver < MP2V.ver) {
    dev[CNAME] = jsl.copyValue(MP2V);
  }
}

evt.onDevDefaults(onDevDefaults);