var createBitsy = require('../lib/createBitsy'),
	Bitsy = createBitsy.Bitsy;

describe('bit getter', function(){
	it('has no side effects', function(){
		var b = createBitsy();
		expect(b.get(0)).toBe(false);
		expect(b.get(99999)).toBe(false);
		expect(b.length).toBe(0);
	});

	it('returns false for anything out of range', function(){
		var b = createBitsy();
		expect(b.get(0)).toBe(false);
		expect(b.get(99999)).toBe(false);
		expect(b.get(1e15)).toBe(false);
		expect(b.get(-1)).toBe(false);
	});

	it('returns true for set values', function(){
		var b = createBitsy(1);
		b.set(0, true);
		expect(b.get(0)).toBe(true);
	});

	it('returns false for unset values', function(){
		var b = createBitsy(1);
		b.set(0, true);
		b.set(0, false);
		expect(b.get(0)).toBe(false);
	});
});

describe('constructor', function(){
	it('can construct large bitsets', function(){
		var MEGABYTE_IN_BITS = 1048576 * 8;
		var bitsy = createBitsy(2 * MEGABYTE_IN_BITS);
		
		expect(bitsy.get(4)).toBe(false);
		bitsy.set(4, true);
		expect(bitsy.get(4)).toBe(true);

		expect(bitsy.get(1 * MEGABYTE_IN_BITS)).toBe(false);
		bitsy.set(1 * MEGABYTE_IN_BITS, true);
		expect(bitsy.get(1 * MEGABYTE_IN_BITS)).toBe(true);
	});
});

describe('toString', function(){
	it('returns hex represenation by default', function(){
		var a = createBitsy(32);
		a.set(0, true);
		a.set(8, true);
		a.set(16, true);
		a.set(24, true);

		expect(a.toString()).toBe('80808080');
	});

	it('can return base 64 representation', function(){
		var a = createBitsy(8);
		
		expect(a.toString('base64')).toBe('AA==');

		a.set(5, true);
		
		expect(a.toString('base64')).toBe('BA==');
	});

	it('can return binary representation', function(){
		var a = createBitsy(8);
		
		a.set(0, true);
		a.set(7, true);

		expect(a.toString('binary')).toBe('10000001');

		a.setSize(16);
		expect(a.toString('binary')).toBe('1000000100000000');

	});
});

describe('slice', function(){
	it('returns new Bitsy with bits from source bitsy', function(){
		var a = createBitsy(1000);
		a.set(10, true);
		var b = a.slice(9,11);

		expect(b instanceof Bitsy).toBe(true);
		expect(b.get(1)).toBe(true);
		expect(b.length).toBe(2);
	});

	it('returns copy when no arguments provided', function(){
		var a = createBitsy(1000);
		a.set(10, true);
		var b = a.slice();

		expect(b instanceof Bitsy).toBe(true);
		expect(b === a).toBe(false);
		expect(b.length).toBe(a.length);
		expect(b.get(10)).toBe(a.get(10));
	});

	it('treats negative start index as "last n elements"', function(){
		var a = createBitsy(15);
		a.set(12, true);
		var b = a.slice(-4); // equal to .slice(10, a.length)

		expect(b instanceof Bitsy).toBe(true);
		expect(b === a).toBe(false);
		expect(b.length).toBe(4); // index 14 - index 10
		expect(b.get(1)).toBe(a.get(12));
	});

	it('treats negative start index as an offset from end', function(){
		var a = createBitsy(15);
		a.set(10, true);
		var b = a.slice(1, -1); // equal to .slice(1, 14)

		expect(b instanceof Bitsy).toBe(true);
		expect(b === a).toBe(false);
		expect(b.length).toBe(13); // index 14 - index 1
		expect(b.get(9)).toBe(a.get(10));
	});
});

describe('copy', function(){
	it('shifts bits when copying', function(){
		var a = createBitsy(1000);
		a.set(100, true);

		var b = createBitsy(50);
		a.copyTo(b, 0, 80, 120);
		
		expect(b.get(20)).toBe(true);
	});
});

describe('bit setter', function(){
	it('handles truthy values as true', function(){
		[true, 1, 42, 'true', 'popcorn'].forEach(function(truthyValue){
			var b = createBitsy(1);
			b.set(0, truthyValue);
			expect(b.get(0)).toBe(true);
		});
	});

	it('setting a value twice has no adverse side-effect', function(){
		var b = createBitsy(1);
		b.set(0, true);
		b.set(0, true);
		expect(b.get(0)).toBe(true);

		var b = createBitsy(1);
		b.set(0, false);
		b.set(0, false);
		expect(b.get(0)).toBe(false);
	});

	it('handles falsy values as false', function(){
		[false, 0, '', null, undefined].forEach(function(falsyValue){
			var b = createBitsy(1);
			b.set(0, true);
			expect(b.get(0)).toBe(true);

			b.set(0, falsyValue);
			expect(b.get(0)).toBe(false);
		});
	});

	it('can set and unset many bits', function(){
		var b = createBitsy(1000);
		var c = 1000;
		while(c--){
			b.set(c, true);
			if(b.get(c) !== true){
				expect(b.get(c)).toBe(true);
				return;
			}
		}

		var c = 1000;
		while(c--){
			b.set(c, false);
			if(b.get(c) !== false){
				expect(b.get(c)).toBe(false);
				break;
			}
		}
	});

	it('can set bits as a sparse map', function(){
		var b = createBitsy(60000);
		var bitIndices = [2, 56000, 400];
		var search = 60000; // exceed the largest set bit index

		var index;
		while(index = bitIndices.pop()){
			b.set(index, true);
			expect(b.get(index)).toBe(true);
		}

		var setBitCount = 0;
		while(search--){
			if(b.get(search)) setBitCount++;
		}

		expect(setBitCount).toBe(3); // because we set 3 indices.
	});
});
